@@ -442,16 +442,15 @@ class FourEyesTokenClass(TokenClass):
         :param transactionid: the id of this challenge
         :param options: the request context parameters / data
         :type options: dict
-        :return: tuple of (bool, message, transactionid, attributes)
+        :return: tuple of (bool, message, transactionid, reply_dict)
         :rtype: tuple
 
         The return tuple builds up like this:
         ``bool`` if submit was successful;
         ``message`` which is displayed in the JSON response;
-        additional ``attributes``, which are displayed in the JSON response.
+        additional challenge ``reply_dict``, which are displayed in the JSON challenges response.
         """
         options = options or {}
-        attributes = {}
         message = ""
         if type(options.get("data")) == dict:
             # In the special first chal-resp case we do not have jsonified data, yet. So we need to convert
@@ -478,8 +477,8 @@ class FourEyesTokenClass(TokenClass):
         db_challenge.save()
         expiry_date = datetime.datetime.now() + \
                       datetime.timedelta(seconds=validity)
-        attributes['valid_until'] = "{0!s}".format(expiry_date)
-        return True, message, db_challenge.transaction_id, attributes
+        reply_dict = {'attributes': {'valid_until': "{0!s}".format(expiry_date)}}
+        return True, message, db_challenge.transaction_id, reply_dict
 
     def is_challenge_request(self, passw, user=None, options=None):
         """
