@@ -29,7 +29,7 @@
 #
 from privacyidea.api.lib.utils import getParam, attestation_certificate_allowed
 from privacyidea.lib.config import get_from_config
-from privacyidea.lib.tokenclass import TokenClass
+from privacyidea.lib.tokenclass import TokenClass, CLIENTMODE, ROLLOUTSTATE
 from privacyidea.lib.token import get_tokens
 from privacyidea.lib.log import log_with
 import logging
@@ -102,6 +102,8 @@ This step returns a serial number.
 
 You need to call the javascript function
 
+.. sourcecode:: javascript
+
     u2f.register([registerRequest], [], function(u2fData) {} );
 
 and the responseHandler needs to send the *clientdata* and *regdata* back to
@@ -209,6 +211,8 @@ class U2fTokenClass(TokenClass):
     The U2F Token implementation.
     """
 
+    client_mode = CLIENTMODE.U2F
+
     @staticmethod
     def get_class_type():
         """
@@ -312,7 +316,6 @@ class U2fTokenClass(TokenClass):
         TokenClass.__init__(self, db_token)
         self.set_type(u"u2f")
         self.hKeyRequired = False
-        self.init_step = 1
 
     def update(self, param, reset_failcount=True):
         """
@@ -323,13 +326,16 @@ class U2fTokenClass(TokenClass):
         :return: None
         """
         TokenClass.update(self, param)
-        description = "U2F initialization"
         reg_data = getParam(param, "regdata")
         verify_cert = is_true(getParam(param, "u2f.verify_cert", default=True))
-        if reg_data:
-            self.init_step = 2
+        if not reg_data:
+            self.token.rollout_state = ROLLOUTSTATE.CLIENTWAIT
+            # Set the description in the first enrollment step
+            if "description" in param:
+                self.set_description(getParam(param, "description", default=""))
+        elif reg_data and self.token.rollout_state == ROLLOUTSTATE.CLIENTWAIT:
             attestation_cert, user_pub_key, key_handle, \
-                signature, description = parse_registration_data(reg_data,
+                signature, automatic_description = parse_registration_data(reg_data,
                                                                  verify_cert=verify_cert)
             client_data = getParam(param, "clientdata", required)
             client_data_str = url_decode(client_data)
@@ -340,7 +346,7 @@ class U2fTokenClass(TokenClass):
                                     user_pub_key, key_handle, signature)
             self.set_otpkey(key_handle)
             self.add_tokeninfo("pubKey", user_pub_key)
-            # add attestation certificat info
+            # add attestation certificate info
             issuer = x509name_to_string(attestation_cert.get_issuer())
             serial = "{!s}".format(attestation_cert.get_serial_number())
             subject = x509name_to_string(attestation_cert.get_subject())
@@ -348,10 +354,14 @@ class U2fTokenClass(TokenClass):
             self.add_tokeninfo("attestation_issuer", issuer)
             self.add_tokeninfo("attestation_serial", serial)
             self.add_tokeninfo("attestation_subject", subject)
-
-        # If a description is given we use the given description
-        description = getParam(param, "description", default=description)
-        self.set_description(description)
+            # Reset rollout state
+            self.token.rollout_state = ""
+            # If no description has already been set, set the automatic description or the
+            # description given in the 2nd request
+            if not self.token.description:
+                self.set_description(getParam(param, "description", default=automatic_description))
+        else:
+            raise ParameterError("regdata provided but token not in clientwait rollout_state.")
 
     @log_with(log)
     def get_init_detail(self, params=None, user=None):
@@ -359,7 +369,8 @@ class U2fTokenClass(TokenClass):
         At the end of the initialization we ask the user to press the button
         """
         response_detail = {}
-        if self.init_step == 1:
+        # get_init_details runs after "update" method. So in the first step clientwait has already been set
+        if self.token.rollout_state == ROLLOUTSTATE.CLIENTWAIT:
             # This is the first step of the init request
             app_id = get_from_config("u2f.appId", "").strip("/")
             from privacyidea.lib.error import TokenAdminError
@@ -374,8 +385,8 @@ class U2fTokenClass(TokenClass):
             response_detail["u2fRegisterRequest"] = register_request
             self.add_tokeninfo("appId", app_id)
 
-        elif self.init_step == 2:
-            # This is the second step of the init request
+        elif self.token.rollout_state == "":
+            # This is the second step of the init request, the clientwait rollout state has been reset
             response_detail["u2fRegisterResponse"] = {"subject":
                                                           self.token.description}
 
@@ -390,13 +401,11 @@ class U2fTokenClass(TokenClass):
 
         At the moment we do not think of other ways to trigger a challenge.
 
-        This function is not decorated with
-            @challenge_response_allowed
+        This function is not decorated with ``@challenge_response_allowed``
         as the U2F token is always a challenge response token!
 
         :param passw: The PIN of the token.
         :param options: dictionary of additional request parameters
-
         :return: returns true or false
         """
         trigger_challenge = False
@@ -425,7 +434,7 @@ class U2fTokenClass(TokenClass):
         The return tuple builds up like this:
         ``bool`` if submit was successful;
         ``message`` which is displayed in the JSON response;
-        additional ``attributes``, which are displayed in the JSON response.
+        additional challenge ``reply_dict``, which are displayed in the JSON challenges response.
         """
         options = options or {}
         message = get_action_values_from_options(SCOPE.AUTH,
@@ -474,16 +483,18 @@ class U2fTokenClass(TokenClass):
                             "keyHandle": key_handle_url}
 
         image_url = IMAGES.get(self.token.description.lower().split()[0], "")
-        response_details = {"u2fSignRequest": u2f_sign_request,
-                            "hideResponseInput": True,
-                            "img": image_url}
+        reply_dict = {"attributes": {"u2fSignRequest": u2f_sign_request,
+                                     "hideResponseInput": self.client_mode != CLIENTMODE.INTERACTIVE,
+                                     "img": image_url},
+                      "image": image_url}
 
-        return True, message, db_challenge.transaction_id, response_details
+        return True, message, db_challenge.transaction_id, reply_dict
 
     @check_token_locked
     def check_otp(self, otpval, counter=None, window=None, options=None):
         """
         This checks the response of a previous challenge.
+
         :param otpval: N/A
         :param counter: The authentication counter
         :param window: N/A
